@@ -3,10 +3,13 @@ Utility functions to return a formatted name and description for a given view.
 """
 from __future__ import unicode_literals
 
+import re
+
+from django.utils.encoding import force_text
 from django.utils.html import escape
 from django.utils.safestring import mark_safe
+
 from rest_framework.compat import apply_markdown
-import re
 
 
 def remove_trailing_string(content, trailing):
@@ -28,14 +31,19 @@ def dedent(content):
     as it fails to dedent multiline docstrings that include
     unindented text on the initial line.
     """
-    whitespace_counts = [len(line) - len(line.lstrip(' '))
-                         for line in content.splitlines()[1:] if line.lstrip()]
+    content = force_text(content)
+    lines = [line for line in content.splitlines()[1:] if line.lstrip()]
 
     # unindent the content if needed
-    if whitespace_counts:
-        whitespace_pattern = '^' + (' ' * min(whitespace_counts))
-        content = re.sub(re.compile(whitespace_pattern, re.MULTILINE), '', content)
-
+    if lines:
+        whitespace_counts = min([len(line) - len(line.lstrip(' ')) for line in lines])
+        tab_counts = min([len(line) - len(line.lstrip('\t')) for line in lines])
+        if whitespace_counts:
+            whitespace_pattern = '^' + (' ' * whitespace_counts)
+            content = re.sub(re.compile(whitespace_pattern, re.MULTILINE), '', content)
+        elif tab_counts:
+            whitespace_pattern = '^' + ('\t' * tab_counts)
+            content = re.sub(re.compile(whitespace_pattern, re.MULTILINE), '', content)
     return content.strip()
 
 
@@ -57,4 +65,5 @@ def markup_description(description):
         description = apply_markdown(description)
     else:
         description = escape(description).replace('\n', '<br />')
+        description = '<p>' + description + '</p>'
     return mark_safe(description)
