@@ -1,14 +1,20 @@
 from __future__ import unicode_literals
+
 import collections
-from rest_framework.compat import OrderedDict, unicode_to_repr
+from collections import OrderedDict
+
+from django.utils.encoding import force_text
+
+from rest_framework.compat import unicode_to_repr
 
 
 class ReturnDict(OrderedDict):
     """
-    Return object from `serialier.data` for the `Serializer` class.
+    Return object from `serializer.data` for the `Serializer` class.
     Includes a backlink to the serializer instance for renderers
     to use if they need richer field information.
     """
+
     def __init__(self, *args, **kwargs):
         self.serializer = kwargs.pop('serializer')
         super(ReturnDict, self).__init__(*args, **kwargs)
@@ -16,17 +22,34 @@ class ReturnDict(OrderedDict):
     def copy(self):
         return ReturnDict(self, serializer=self.serializer)
 
+    def __repr__(self):
+        return dict.__repr__(self)
+
+    def __reduce__(self):
+        # Pickling these objects will drop the .serializer backlink,
+        # but preserve the raw data.
+        return (dict, (dict(self),))
+
 
 class ReturnList(list):
     """
-    Return object from `serialier.data` for the `SerializerList` class.
+    Return object from `serializer.data` for the `SerializerList` class.
     Includes a backlink to the serializer instance for renderers
     to use if they need richer field information.
     """
+
     def __init__(self, *args, **kwargs):
         self.serializer = kwargs.pop('serializer')
         super(ReturnList, self).__init__(*args, **kwargs)
 
+    def __repr__(self):
+        return list.__repr__(self)
+
+    def __reduce__(self):
+        # Pickling these objects will drop the .serializer backlink,
+        # but preserve the raw data.
+        return (list, (list(self),))
+
 
 class BoundField(object):
     """
@@ -34,8 +57,10 @@ class BoundField(object):
     Returned when iterating over a serializer instance,
     providing an API similar to Django forms and form fields.
     """
+
     def __init__(self, field, value, errors, prefix=''):
         self._field = field
+        self._prefix = prefix
         self.value = value
         self.errors = errors
         self.name = prefix + self.field_name
@@ -52,6 +77,10 @@ class BoundField(object):
             self.__class__.__name__, self.value, self.errors
         ))
 
+    def as_form_field(self):
+        value = '' if (self.value is None or self.value is False) else self.value
+        return self.__class__(self._field, value, self.errors, self._prefix)
+
 
 class NestedBoundField(BoundField):
     """
@@ -59,6 +88,12 @@ class NestedBoundField(BoundField):
     in order to support nested bound fields. This class is the type of
     `BoundField` that is used for serializer fields.
     """
+
+    def __init__(self, field, value, errors, prefix=''):
+        if value is None or value is '':
+            value = {}
+        super(NestedBoundField, self).__init__(field, value, errors, prefix)
+
     def __iter__(self):
         for field in self.fields.values():
             yield self[field.field_name]
@@ -71,6 +106,15 @@ class NestedBoundField(BoundField):
             return NestedBoundField(field, value, error, prefix=self.name + '.')
         return BoundField(field, value, error, prefix=self.name + '.')
 
+    def as_form_field(self):
+        values = {}
+        for key, value in self.value.items():
+            if isinstance(value, (list, dict)):
+                values[key] = value
+            else:
+                values[key] = '' if (value is None or value is False) else force_text(value)
+        return self.__class__(self._field, values, self.errors, self._prefix)
+
 
 class BindingDict(collections.MutableMapping):
     """
@@ -80,6 +124,7 @@ class BindingDict(collections.MutableMapping):
     `field.bind()` so that the `field_name` and `parent` attributes
     can be set correctly.
     """
+
     def __init__(self, serializer):
         self.serializer = serializer
         self.fields = OrderedDict()
@@ -99,3 +144,6 @@ class BindingDict(collections.MutableMapping):
 
     def __len__(self):
         return len(self.fields)
+
+    def __repr__(self):
+        return dict.__repr__(self.fields)
