@@ -7,13 +7,34 @@ object creation, and makes it possible to switch between using the implicit
 `ModelSerializer` class and an equivalent explicit `Serializer` class.
 """
 from __future__ import unicode_literals
+
+from django.db import DataError
 from django.utils.translation import ugettext_lazy as _
+
 from rest_framework.compat import unicode_to_repr
 from rest_framework.exceptions import ValidationError
 from rest_framework.utils.representation import smart_repr
 
 
-class UniqueValidator:
+# Robust filter and exist implementations. Ensures that queryset.exists() for
+# an invalid value returns `False`, rather than raising an error.
+# Refs https://github.com/tomchristie/django-rest-framework/issues/3381
+
+def qs_exists(queryset):
+    try:
+        return queryset.exists()
+    except (TypeError, ValueError, DataError):
+        return False
+
+
+def qs_filter(queryset, **kwargs):
+    try:
+        return queryset.filter(**kwargs)
+    except (TypeError, ValueError, DataError):
+        return queryset.none()
+
+
+class UniqueValidator(object):
     """
     Validator that corresponds to `unique=True` on a model field.
 
@@ -21,10 +42,11 @@ class UniqueValidator:
     """
     message = _('This field must be unique.')
 
-    def __init__(self, queryset, message=None):
+    def __init__(self, queryset, message=None, lookup='exact'):
         self.queryset = queryset
         self.serializer_field = None
         self.message = message or self.message
+        self.lookup = lookup
 
     def set_context(self, serializer_field):
         """
@@ -33,7 +55,7 @@ class UniqueValidator:
         """
         # Determine the underlying model field name. This may not be the
         # same as the serializer field name if `source=<>` is set.
-        self.field_name = serializer_field.source_attrs[0]
+        self.field_name = serializer_field.source_attrs[-1]
         # Determine the existing instance, if this is an update operation.
         self.instance = getattr(serializer_field.parent, 'instance', None)
 
@@ -41,8 +63,8 @@ class UniqueValidator:
         """
         Filter the queryset to all instances matching the given attribute.
         """
-        filter_kwargs = {self.field_name: value}
-        return queryset.filter(**filter_kwargs)
+        filter_kwargs = {'%s__%s' % (self.field_name, self.lookup): value}
+        return qs_filter(queryset, **filter_kwargs)
 
     def exclude_current_instance(self, queryset):
         """
@@ -57,8 +79,8 @@ class UniqueValidator:
         queryset = self.queryset
         queryset = self.filter_queryset(value, queryset)
         queryset = self.exclude_current_instance(queryset)
-        if queryset.exists():
-            raise ValidationError(self.message)
+        if qs_exists(queryset):
+            raise ValidationError(self.message, code='unique')
 
     def __repr__(self):
         return unicode_to_repr('<%s(queryset=%s)>' % (
@@ -67,7 +89,7 @@ class UniqueValidator:
         ))
 
 
-class UniqueTogetherValidator:
+class UniqueTogetherValidator(object):
     """
     Validator that corresponds to `unique_together = (...)` on a model class.
 
@@ -98,13 +120,13 @@ class UniqueTogetherValidator:
         if self.instance is not None:
             return
 
-        missing = dict([
-            (field_name, self.missing_message)
+        missing_items = {
+            field_name: self.missing_message
             for field_name in self.fields
             if field_name not in attrs
-        ])
-        if missing:
-            raise ValidationError(missing)
+        }
+        if missing_items:
+            raise ValidationError(missing_items, code='required')
 
     def filter_queryset(self, attrs, queryset):
         """
@@ -118,11 +140,11 @@ class UniqueTogetherValidator:
                     attrs[field_name] = getattr(self.instance, field_name)
 
         # Determine the filter keyword arguments and filter the queryset.
-        filter_kwargs = dict([
-            (field_name, attrs[field_name])
+        filter_kwargs = {
+            field_name: attrs[field_name]
             for field_name in self.fields
-        ])
-        return queryset.filter(**filter_kwargs)
+        }
+        return qs_filter(queryset, **filter_kwargs)
 
     def exclude_current_instance(self, attrs, queryset):
         """
@@ -138,9 +160,15 @@ class UniqueTogetherValidator:
         queryset = self.queryset
         queryset = self.filter_queryset(attrs, queryset)
         queryset = self.exclude_current_instance(attrs, queryset)
-        if queryset.exists():
+
+        # Ignore validation if any field is None
+        checked_values = [
+            value for field, value in attrs.items() if field in self.fields
+        ]
+        if None not in checked_values and qs_exists(queryset):
             field_names = ', '.join(self.fields)
-            raise ValidationError(self.message.format(field_names=field_names))
+            message = self.message.format(field_names=field_names)
+            raise ValidationError(message, code='unique')
 
     def __repr__(self):
         return unicode_to_repr('<%s(queryset=%s, fields=%s)>' % (
@@ -150,7 +178,7 @@ class UniqueTogetherValidator:
         ))
 
 
-class BaseUniqueForValidator:
+class BaseUniqueForValidator(object):
     message = None
     missing_message = _('This field is required.')
 
@@ -167,8 +195,8 @@ class BaseUniqueForValidator:
         """
         # Determine the underlying model field names. These may not be the
         # same as the serializer field names if `source=<>` is set.
-        self.field_name = serializer.fields[self.field].source_attrs[0]
-        self.date_field_name = serializer.fields[self.date_field].source_attrs[0]
+        self.field_name = serializer.fields[self.field].source_attrs[-1]
+        self.date_field_name = serializer.fields[self.date_field].source_attrs[-1]
         # Determine the existing instance, if this is an update operation.
         self.instance = getattr(serializer, 'instance', None)
 
@@ -177,13 +205,13 @@ class BaseUniqueForValidator:
         The `UniqueFor<Range>Validator` classes always force an implied
         'required' state on the fields they are applied to.
         """
-        missing = dict([
-            (field_name, self.missing_message)
+        missing_items = {
+            field_name: self.missing_message
             for field_name in [self.field, self.date_field]
             if field_name not in attrs
-        ])
-        if missing:
-            raise ValidationError(missing)
+        }
+        if missing_items:
+            raise ValidationError(missing_items, code='required')
 
     def filter_queryset(self, attrs, queryset):
         raise NotImplementedError('`filter_queryset` must be implemented.')
@@ -202,9 +230,11 @@ class BaseUniqueForValidator:
         queryset = self.queryset
         queryset = self.filter_queryset(attrs, queryset)
         queryset = self.exclude_current_instance(attrs, queryset)
-        if queryset.exists():
+        if qs_exists(queryset):
             message = self.message.format(date_field=self.date_field)
-            raise ValidationError({self.field: message})
+            raise ValidationError({
+                self.field: message
+            }, code='unique')
 
     def __repr__(self):
         return unicode_to_repr('<%s(queryset=%s, field=%s, date_field=%s)>' % (
@@ -227,7 +257,7 @@ class UniqueForDateValidator(BaseUniqueForValidator):
         filter_kwargs['%s__day' % self.date_field_name] = date.day
         filter_kwargs['%s__month' % self.date_field_name] = date.month
         filter_kwargs['%s__year' % self.date_field_name] = date.year
-        return queryset.filter(**filter_kwargs)
+        return qs_filter(queryset, **filter_kwargs)
 
 
 class UniqueForMonthValidator(BaseUniqueForValidator):
@@ -240,7 +270,7 @@ class UniqueForMonthValidator(BaseUniqueForValidator):
         filter_kwargs = {}
         filter_kwargs[self.field_name] = value
         filter_kwargs['%s__month' % self.date_field_name] = date.month
-        return queryset.filter(**filter_kwargs)
+        return qs_filter(queryset, **filter_kwargs)
 
 
 class UniqueForYearValidator(BaseUniqueForValidator):
@@ -253,4 +283,4 @@ class UniqueForYearValidator(BaseUniqueForValidator):
         filter_kwargs = {}
         filter_kwargs[self.field_name] = value
         filter_kwargs['%s__year' % self.date_field_name] = date.year
-        return queryset.filter(**filter_kwargs)
+        return qs_filter(queryset, **filter_kwargs)
